#ifndef PEBBLISP_H
#define PEBBLISP_H

#include <stdio.h>

#include "env.h"
#include "object.h"

#define F(_name, _docs) static const char * const _name ## Doc = _docs; Object _name

struct Slice {
    const char* text;
    unsigned char length;
};

typedef struct Result {
    Object obj;
    struct Slice* slices;
} Result;

Object eval(const Object* obj, struct Environment* env);

Result parse(struct Slice* slices);

Result readSeq(struct Slice* slices);

Result parseAtom(struct Slice* slice);

Object parseEval(const char* input, struct Environment* env);

void evalForms(Object* destList, const Object* src, struct Environment* env);

void copySlice(char* dest, struct Slice* src);

Object evalLambdaArgs(const Object* arg_forms, struct Environment* env);

Object listEvalLambda(Object* lambda, const Object* remaining,
                      struct Environment* env);

void debugSlice(struct Slice* s);

Object simpleFuncEval(Object func, Object arg1, Object arg2, struct Environment* env);

#ifdef STANDALONE

Object takeInput(Object i1, Object i2, struct Environment* i3);

Object systemCall(Object call, Object _, struct Environment* i3);

Object loadFile(Object filename, Object _, struct Environment* env);

int _readFile(FILE* input, struct Environment* env);

int readFile(const char* filename, struct Environment* env);

#endif /* STANDALONE */

#endif