#ifndef PEBBLISP_PLFUNC_H
#define PEBBLISP_PLFUNC_H

#include "pebblisp.h"

#define BASIC_OP(_name) \
    Object _name(Object obj1, Object obj2, struct Environment *env);

BASIC_OP(add);

BASIC_OP(sub);

BASIC_OP(mul);

BASIC_OP(dvi);

BASIC_OP(mod);

BASIC_OP(equ);

BASIC_OP(gth);

BASIC_OP(lth);

BASIC_OP(and);

BASIC_OP(or);

#undef BASIC_OP

F(catObjects,
  "Concatenate string versions of the given objects.\n"
  "(cat \"Stuff: \" (1 2 3)) => \"Stuff: ( 1 2 3 )\""
)(Object obj1, Object obj2, struct Environment* env);

F(filter, "(filter (< 50) (25 60 100)) => ( 60 100 )")
(Object condition, Object list, struct Environment* env);

F(append, "(ap (1 2) 3) => ( 1 2 3 )")
(Object list, Object newElement, struct Environment* env);

F(prepend, "(pre (2 3) 1) => ( 1 2 3 )")
(Object list, Object newElement, struct Environment* env);

F(len, "(len (2 3)) => 2")
(Object obj1, Object o_ignore, struct Environment* e_ignore);

F(reduce, "(reduce ((1 2 3) 0) +) => 6")
(Object listInitial, Object func, struct Environment* env);

F(at, "(at 1 (1 2 3)) => 2")
(Object index, Object list, struct Environment* env);

F(rest, "(rest (1 2 3)) => ( 2 3 )")
(Object list, Object ignore, struct Environment* env);

F(reverse, "(rev (1 2 3)) => ( 3 2 1 )")
(Object _list, Object ignore, struct Environment* ignore2);

F(isNum, "(isnum 1) => T\n(isnum \"Hello\") => F")
(Object test, Object ignore, struct Environment* ignore2);

F(isList,
  "(islist (1 2 3)) => T\n"
  "(islist ()) => T\n"
  "(islist \"Stringy\") => F"
)(Object test, Object ignore, struct Environment* ignore2);

F(isString,
  "(isstr \"Heyo\") => T\n"
  "(isstr \"\") => T\n"
  "(isstr 10) => F"
)(Object test, Object ignore, struct Environment* ignore2);

F(isErr, "")
(Object test, Object ignore, struct Environment* ignore2);

F(charAt,
  "(chat \"Hello\" 1) => \"e\"\n"
  "(chat \"Hello\" 10) => \"\""
)(Object string, Object at, struct Environment* ignore);

F(charVal,
  "(char \"h\") => 104\n"
  "(char \"hello\") => 104\n"
  "(char \"\") => 0"
)(Object test, Object ignore, struct Environment* ignore2);

F(parseEvalO, "(eval \"(1 2 3)\") => (1 2 3)")
(Object text, Object ignore, struct Environment* env);

F(possessive,
  "(struct Post (title body)) (def p (Post \"TI\" \"BO\")) p's title => TI"
)(Object structo, Object field, struct Environment* env);

#ifdef STANDALONE

F(print, "Prints the string representation of the given object to stdout.")
(Object p, Object ignore, struct Environment* ignore2);

F(pChar, "Prints the ascii character for the given number value.")
(Object c, Object i1, struct Environment* i2);

F(printEnvO, "Prints out the current scoped environment.")
(Object i1, Object i2, struct Environment* env);

F(help, "(? +) => \"(+ 1 2) => 3\"")
(Object symbol, Object ignore, struct Environment* ignore2);

#endif

#endif //PEBBLISP_PLFUNC_H
